package sun.swing;

import javax.swing.JComponent;
import java.awt.dnd.DropTarget;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.dnd.peer.DragSourceContextPeer;
import java.awt.dnd.DragGestureEvent;
import java.awt.dnd.DragGestureRecognizer;
import java.awt.dnd.DragGestureListener;
import java.awt.dnd.DragSource;
import java.awt.peer.ComponentPeer;
import sun.awt.OverrideNativeWindowHandle;
import sun.awt.AWTAccessor;
import java.awt.MouseInfo;
import javax.swing.JLayeredPane;
import java.awt.Container;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;
import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.image.DataBufferInt;
import sun.awt.DisplayChangedListener;
import java.awt.Graphics2D;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Window;
import javax.swing.RepaintManager;
import java.awt.Rectangle;
import javax.swing.SwingUtilities;
import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.awt.Color;
import javax.swing.LayoutFocusTraversalPolicy;
import java.security.AccessController;
import sun.security.action.GetPropertyAction;
import java.beans.PropertyChangeListener;
import java.awt.image.BufferedImage;
import javax.swing.JPanel;
import java.awt.Component;
import javax.swing.JRootPane;
import javax.swing.RootPaneContainer;
import sun.awt.LightweightFrame;

public final class JLightweightFrame extends LightweightFrame implements RootPaneContainer
{
    private final JRootPane rootPane;
    private LightweightContent content;
    private Component component;
    private JPanel contentPane;
    private BufferedImage bbImage;
    private volatile int scaleFactor;
    private static boolean copyBufferEnabled;
    private int[] copyBuffer;
    private PropertyChangeListener layoutSizeListener;
    private SwingUtilities2.RepaintListener repaintListener;
    
    public JLightweightFrame() {
        this.rootPane = new JRootPane();
        this.scaleFactor = 1;
        JLightweightFrame.copyBufferEnabled = "true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.jlf.copyBufferEnabled", "true")));
        this.add(this.rootPane, "Center");
        this.setFocusTraversalPolicy(new LayoutFocusTraversalPolicy());
        if (this.getGraphicsConfiguration().isTranslucencyCapable()) {
            this.setBackground(new Color(0, 0, 0, 0));
        }
        this.layoutSizeListener = new PropertyChangeListener() {
            @Override
            public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
                Dimension dimension = (Dimension)propertyChangeEvent.getNewValue();
                if ("preferredSize".equals(propertyChangeEvent.getPropertyName())) {
                    JLightweightFrame.this.content.preferredSizeChanged(dimension.width, dimension.height);
                }
                else if ("maximumSize".equals(propertyChangeEvent.getPropertyName())) {
                    JLightweightFrame.this.content.maximumSizeChanged(dimension.width, dimension.height);
                }
                else if ("minimumSize".equals(propertyChangeEvent.getPropertyName())) {
                    JLightweightFrame.this.content.minimumSizeChanged(dimension.width, dimension.height);
                }
            }
        };
        this.repaintListener = ((component2, x, y, width, height)/* javax.swing.JComponent,int,int,int,int, */ -> {
            Window destination = SwingUtilities.getWindowAncestor(component2);
            if (destination != this) {
                return;
            }
            Point point = SwingUtilities.convertPoint(component2, x, y, destination);
            Rectangle rectangle = new Rectangle(point.x, point.y, width, height).intersection(new Rectangle(0, 0, this.bbImage.getWidth() / this.scaleFactor, this.bbImage.getHeight() / this.scaleFactor));
            if (!rectangle.isEmpty()) {
                this.notifyImageUpdated(rectangle.x, rectangle.y, rectangle.width, rectangle.height);
            }
            return;
        });
        SwingAccessor.getRepaintManagerAccessor().addRepaintListener(RepaintManager.currentManager(this), this.repaintListener);
    }
    
    @Override
    public void dispose() {
        SwingAccessor.getRepaintManagerAccessor().removeRepaintListener(RepaintManager.currentManager(this), this.repaintListener);
        super.dispose();
    }
    
    public void setContent(LightweightContent content) {
        if (content == null) {
            System.err.println("JLightweightFrame.setContent: content may not be null!");
            return;
        }
        this.content = content;
        this.component = content.getComponent();
        Dimension preferredSize = this.component.getPreferredSize();
        content.preferredSizeChanged(preferredSize.width, preferredSize.height);
        Dimension maximumSize = this.component.getMaximumSize();
        content.maximumSizeChanged(maximumSize.width, maximumSize.height);
        Dimension minimumSize = this.component.getMinimumSize();
        content.minimumSizeChanged(minimumSize.width, minimumSize.height);
        this.initInterior();
        return;
    }
    
    @Override
    public Graphics getGraphics() {
        if (this.bbImage == null) {
            return null;
        }
        Graphics2D graphics = this.bbImage.createGraphics();
        graphics.setBackground(this.getBackground());
        graphics.setColor(this.getForeground());
        graphics.setFont(this.getFont());
        graphics.scale((double)this.scaleFactor, (double)this.scaleFactor);
        return graphics;
    }
    
    @Override
    public void grabFocus() {
        if (this.content != null) {
            this.content.focusGrabbed();
        }
    }
    
    @Override
    public void ungrabFocus() {
        if (this.content != null) {
            this.content.focusUngrabbed();
        }
    }
    
    @Override
    public int getScaleFactor() {
        return this.scaleFactor;
    }
    
    @Override
    public void notifyDisplayChanged(int scaleFactor) {
        if (scaleFactor != this.scaleFactor) {
            if (!JLightweightFrame.copyBufferEnabled) {
                this.content.paintLock();
            }
            try {
                if (this.bbImage != null) {
                    this.resizeBuffer(this.getWidth(), this.getHeight(), scaleFactor);
                }
            }
            finally {
                if (!JLightweightFrame.copyBufferEnabled) {
                    this.content.paintUnlock();
                }
            }
            this.scaleFactor = scaleFactor;
        }
        if ((this.getPeer() instanceof DisplayChangedListener)) {
            ((DisplayChangedListener)this.getPeer()).displayChanged();
        }
        this.repaint();
    }
    
    @Override
    public void addNotify() {
        super.addNotify();
        if ((this.getPeer() instanceof DisplayChangedListener)) {
            ((DisplayChangedListener)this.getPeer()).displayChanged();
        }
    }
    
    private void syncCopyBuffer(boolean b, int n, int n2, int n3, int n4, int n5) {
        this.content.paintLock();
        try {
            int[] data = ((DataBufferInt)this.bbImage.getRaster().getDataBuffer()).getData();
            if (b) {
                this.copyBuffer = new int[data.length];
            }
            int width = this.bbImage.getWidth();
            n *= n5;
            n2 *= n5;
            n3 *= n5;
            n4 *= n5;
            for (int i = 0; i < n4; ++i) {
                int n6 = (n2 + i) * width + n;
                System.arraycopy(data, n6, this.copyBuffer, n6, n3);
            }
        }
        finally {
            this.content.paintUnlock();
        }
    }
    
    private void notifyImageUpdated(int n, int n2, int n3, int n4) {
        if (JLightweightFrame.copyBufferEnabled) {
            this.syncCopyBuffer(false, n, n2, n3, n4, this.scaleFactor);
        }
        this.content.imageUpdated(n, n2, n3, n4);
    }
    
    private void initInterior() {
        (this.contentPane = new JPanel() {
            @Override
            public void paint(Graphics g) {
                if (!JLightweightFrame.copyBufferEnabled) {
                    JLightweightFrame.this.content.paintLock();
                }
                try {
                    super.paint(g);
                    Rectangle rectangle = (g.getClipBounds() != null) ? g.getClipBounds() : new Rectangle(0, 0, JLightweightFrame.this.contentPane.getWidth(), JLightweightFrame.this.contentPane.getHeight());
                    rectangle.x = Math.max(0, rectangle.x);
                    rectangle.y = Math.max(0, rectangle.y);
                    rectangle.width = Math.min(JLightweightFrame.this.contentPane.getWidth(), rectangle.width);
                    rectangle.height = Math.min(JLightweightFrame.this.contentPane.getHeight(), rectangle.height);
                    EventQueue.invokeLater(new Runnable() {
                        @Override
                        public void run() {
                            Rectangle intersection = JLightweightFrame.this.contentPane.getBounds().intersection(rectangle);
                            JLightweightFrame.access$400(JLightweightFrame.this, intersection.x, intersection.y, intersection.width, intersection.height);
                        }
                    });
                }
                finally {
                    if (!JLightweightFrame.copyBufferEnabled) {
                        JLightweightFrame.this.content.paintUnlock();
                    }
                }
            }
            
            @Override
            protected boolean isPaintingOrigin() {
                return true;
            }
        }).setLayout(new BorderLayout());
        this.contentPane.add(this.component);
        if ("true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.jlf.contentPaneTransparent", "false")))) {
            this.contentPane.setOpaque(false);
        }
        this.setContentPane(this.contentPane);
        this.contentPane.addContainerListener(new ContainerListener() {
            @Override
            public void componentAdded(ContainerEvent containerEvent) {
                Component access$500 = JLightweightFrame.this.component;
                if (containerEvent.getChild() == access$500) {
                    access$500.addPropertyChangeListener("preferredSize", JLightweightFrame.this.layoutSizeListener);
                    access$500.addPropertyChangeListener("maximumSize", JLightweightFrame.this.layoutSizeListener);
                    access$500.addPropertyChangeListener("minimumSize", JLightweightFrame.this.layoutSizeListener);
                }
            }
            
            @Override
            public void componentRemoved(ContainerEvent containerEvent) {
                Component access$500 = JLightweightFrame.this.component;
                if (containerEvent.getChild() == access$500) {
                    access$500.removePropertyChangeListener(JLightweightFrame.this.layoutSizeListener);
                }
            }
        });
    }
    
    @Override
    public void reshape(int x, int y, int max, int max2) {
        super.reshape(x, y, max, max2);
        if (max == 0 || max2 == 0) {
            return;
        }
        if (!JLightweightFrame.copyBufferEnabled) {
            this.content.paintLock();
        }
        try {
            int n = this.bbImage == null;
            if (this.bbImage != null) {
                int n2 = this.bbImage.getWidth() / this.scaleFactor;
                int n3 = this.bbImage.getHeight() / this.scaleFactor;
                if (max != n2 || max2 != n3) {
                    n = true;
                    if (this.bbImage != null) {
                        if (n2 >= max && n3 >= max2) {
                            n = false;
                        }
                        else {
                            if (n2 >= max) {
                                max = n2;
                            }
                            else {
                                max = Math.max((int)((double)n2 * 1.2), max);
                            }
                            if (n3 >= max2) {
                                max2 = n3;
                            }
                            else {
                                max2 = Math.max((int)((double)n3 * 1.2), max2);
                            }
                        }
                    }
                }
            }
            if (!(n == 0)) {
                this.resizeBuffer(max, max2, this.scaleFactor);
                return;
            }
            this.content.imageReshaped(0, 0, max, max2);
        }
        finally {
            if (!JLightweightFrame.copyBufferEnabled) {
                this.content.paintUnlock();
            }
        }
        return;
    }
    
    private void resizeBuffer(int n, int n2, int n3) {
        this.bbImage = new BufferedImage(n * n3, n2 * n3, 3);
        int[] array = ((DataBufferInt)this.bbImage.getRaster().getDataBuffer()).getData();
        if (JLightweightFrame.copyBufferEnabled) {
            this.syncCopyBuffer(true, 0, 0, n, n2, n3);
            array = this.copyBuffer;
        }
        this.content.imageBufferReset(array, 0, 0, n, n2, n * n3, n3);
    }
    
    @Override
    public JRootPane getRootPane() {
        return this.rootPane;
    }
    
    @Override
    public void setContentPane(Container contentPane) {
        this.getRootPane().setContentPane(contentPane);
    }
    
    @Override
    public Container getContentPane() {
        return this.getRootPane().getContentPane();
    }
    
    @Override
    public void setLayeredPane(JLayeredPane layeredPane) {
        this.getRootPane().setLayeredPane(layeredPane);
    }
    
    @Override
    public JLayeredPane getLayeredPane() {
        return this.getRootPane().getLayeredPane();
    }
    
    @Override
    public void setGlassPane(Component glassPane) {
        this.getRootPane().setGlassPane(glassPane);
    }
    
    @Override
    public Component getGlassPane() {
        return this.getRootPane().getGlassPane();
    }
    
    private void updateClientCursor() {
        Point location = MouseInfo.getPointerInfo().getLocation();
        SwingUtilities.convertPointFromScreen(location, this);
        Component deepestComponent = SwingUtilities.getDeepestComponentAt(this, location.x, location.y);
        if (deepestComponent != null) {
            this.content.setCursor(deepestComponent.getCursor());
        }
    }
    
    public void overrideNativeWindowHandle(long n, Runnable runnable) {
        ComponentPeer peer = AWTAccessor.getComponentAccessor().getPeer(this);
        if ((peer instanceof OverrideNativeWindowHandle)) {
            ((OverrideNativeWindowHandle)peer).overrideWindowHandle(n);
        }
        if (runnable != null) {
            runnable.run();
        }
    }
    
    @Override
    public <T extends DragGestureRecognizer> T createDragGestureRecognizer(Class<T> clazz, DragSource dragSource, Component component, int n, DragGestureListener dragGestureListener) {
        return (this.content == null) ? null : this.content.createDragGestureRecognizer(clazz, dragSource, component, n, dragGestureListener);
    }
    
    @Override
    public DragSourceContextPeer createDragSourceContextPeer(DragGestureEvent dragGestureEvent) throws InvalidDnDOperationException {
        return (this.content == null) ? null : this.content.createDragSourceContextPeer(dragGestureEvent);
    }
    
    @Override
    public void addDropTarget(DropTarget dropTarget) {
        if (this.content == null) {
            return;
        }
        this.content.addDropTarget(dropTarget);
        return;
    }
    
    @Override
    public void removeDropTarget(DropTarget dropTarget) {
        if (this.content == null) {
            return;
        }
        this.content.removeDropTarget(dropTarget);
        return;
    }
    
    static void access$000(JLightweightFrame lightweightFrame) {
        lightweightFrame.updateClientCursor();
    }
    
    static void access$400(JLightweightFrame lightweightFrame, int n, int n2, int n3, int n4) {
        lightweightFrame.notifyImageUpdated(n, n2, n3, n4);
    }
    
    static {
        SwingAccessor.setJLightweightFrameAccessor(new SwingAccessor.JLightweightFrameAccessor() {
            @Override
            public void updateCursor(JLightweightFrame lightweightFrame) {
                JLightweightFrame.access$000(lightweightFrame);
            }
        });
        JLightweightFrame.copyBufferEnabled = "true".equals(AccessController.doPrivileged(new GetPropertyAction("swing.jlf.copyBufferEnabled", "true")));
    }
}
